import clingo
import copy 
import time
import sys
import re

# Observer to track: minimize literals
class Observer:
    def __init__(self):
        self.minimize_literals = {}
        self.priorities = []

    def minimize(self, priority, literals):
        self.priorities.append(priority)
        self.minimize_literals[priority] = literals

class Application:
    def __init__(self, name):
        self.program_name = name
        self.my_models = []
        self.result_dict = {}
        self.opt_times = []

    def main(self, ctl, files):
        if len(files) > 0:
            for f in files:
                ctl.load(f)
        else:
            ctl.load("-")

        # ===================================
        # Initialize
        assert(ctl.configuration.solve.opt_mode == 'optN'), 'Set --opt-mode=OptN.'
        count = int(ctl.configuration.solve.models)
        start = time.time()

        # register observer
        obs = Observer()
        ctl.register_observer(obs)
        solutions = []
    
        # ===================================
        # Ground and Solve
        ctl.ground([("base", [])])
        enumerated = False
        while count > 0 and not enumerated:
            costs = []   
            ctl.configuration.solve.models = count
            with ctl.solve(yield_=True) as handle:
                for model in handle:
                    if model.optimality_proven:
                        costs = model.cost[:]
                        self.result_dict[costs[0]] = self.result_dict.get(costs[0], 0) + 1
                        self.my_models.append((costs, str(model)))
                        self.opt_times.append(time.time() - start)
                        count -= 1
                # Everything was enumerated      
                if handle.get().unsatisfiable:
                    enumerated = True
                    break
            # If we want to find more answer sets, enumerate on weight
            if count > 0:
                with ctl.backend() as backend: 
                    for i in range(len(obs.priorities)):
                        aux = ctl.backend().add_atom() 
                        ctl.backend().add_weight_rule([aux], costs[i] + 1, 
                            obs.minimize_literals[obs.priorities[i]])
                        ctl.backend().add_rule([], [-aux])


        enumerated = True
        # return solutions