
import clingo
import time 
import bisect

class Observer:
    def __init__(self):
        self.minimize_literals = {}
        self.priorities = []
        self.current_assignment = []

        self.symbolic_cost_map = {}
        self.cost_map = {}
        self.lookup = []

        self.topN = []
        self.assN = []
        self.N = 1

        self.model_times = []

    # Keep track of the minimize-literals
    def minimize(self, priority, literals):
        if priority not in self.minimize_literals.keys():
            self.minimize_literals[priority] = []
            self.priorities.append(priority)
        self.minimize_literals[priority] = literals

        for literal, cost in literals:
            self.symbolic_cost_map[literal] = cost

    def found_model(self, cost_sum, model):
        #cost_sum = 0
        #for lit in self.current_assignment:
        #    if lit in self.cost_map.keys():
        #        cost_sum += self.cost_map[lit]

        if len(self.topN) < self.N or self.topN[self.N - 1] > cost_sum:
            if len(self.topN) == self.N:
                self.topN = self.topN[:-1]
                self.assN = self.assN[:-1]
            self.solve_top_list(cost_sum, model)
# 73!
    def solve_top_list(self, cost_sum, model):
        bisect.insort(self.topN, cost_sum) 
        bisect.insort(self.assN, (cost_sum, model))

        
    def init(self,  init):
        
        for l in self.symbolic_cost_map.keys():
            lit = init.solver_literal(l)
            if lit not in self.cost_map.keys():
                self.cost_map[lit] = self.symbolic_cost_map[l]
            else:
                self.cost_map[lit] += self.symbolic_cost_map[l]
            init.add_watch(lit)      

    def propagate(self, control, changes):
        for lit in changes:
            self.current_assignment.append(lit)
            
        cost_sum = 0
        for lit in self.current_assignment:
            if lit in self.cost_map.keys():
                cost_sum += self.cost_map[lit]
        
        # If cost succeeds boundaries, we can add the model to nogoods
        if len(self.topN) == self.N and self.topN[self.N - 1] < cost_sum: 
            control.add_nogood(self.current_assignment, lock=1)
        
    def undo(self,  thread_id,  assignment, changes):
        for lit in changes:
            self.current_assignment.remove(lit)
    


class Sapplication:
    def __init__(self, name):
        self.program_name = name
        self.my_models = []
        self.result_dict = {}

    def main(self, ctl, files):
        if len(files) > 0:
            for f in files:
                ctl.load(f)
        else:
            ctl.load("-")

        start = time.time()

        assert(ctl.configuration.solve.opt_mode == 'enum'), 'Set --opt-mode=enum'
        start = time.time()

        # Use model count as number of models to output
        count = int(ctl.configuration.solve.models)
        assert count > 0, "need to know window size for smart enum"
        ctl.configuration.solve.models = 0

        # register observer
        obs = Observer()
        obs.N = count
        ctl.register_observer(obs)
        ctl.register_propagator(obs)
        
        ctl.ground([('base', [])])
        solutions = []

        with ctl.solve(yield_=True) as handle:
            for model in handle:
                # Calculate cost for found models
                obs.found_model(model.cost[0], str(model))

        self.solutions = obs.topN # solutions
        self.my_models = obs.assN[:]
        
        for cost in obs.topN:
            self.result_dict[cost] = self.result_dict.get(cost, 0) + 1
        
        self.enumerated = True
