import os
import sys
import time
import clingo
import pickle
import numpy as np
import matplotlib.pyplot as plt

# Baseline, enumerates all of the solutions
from ASPalgorithms.enum_application import enumication

# We only need to run the baseline for every instance once
def baseline(main_file, instance, time_limit=600):
    
    start = time.time()
    clingo_argumenst = [main_file, instance, '-t 2','--opt-mode=enum', 0, '--time-limit={}'.format(time_limit), "--quiet=1"]    
    enum_baseline = enumication(sys.argv[0])
    clingo.clingo_main(enum_baseline, clingo_argumenst)
    end = time.time()
    
    result = {}
    result["weights"] = enum_baseline.result_dict
    result["completed"] = False if time_limit - (end - start) < 0.5 else True
    result["time"] = end - start
    return result 
    
    

main_files = ['ConnectedMaximim-densityStillLife/encoding.asp',
              'Hamilton/tsp.asp',
              'FastFood/FastfoodOptimize.asp',
              'BayesianNL/encoding.asp', 
              'MarkovNL/encoding.asp',
              'Supertree/encoding.asp',
              'MaxSat/encoding-show.asp']
directorys = ['ConnectedMaximim-densityStillLife/',
              'Hamilton/instances',
              'FastFood/instances',
              'BayesianNL/instances', 
              'MarkovNL/instances',
              'Supertree/instances',
              'MaxSat/']


result_path = "Results/"

def main():
    for main_file, directory in zip(main_files, directorys):
        if not os.path.isfile(result_path + main_file.split('/')[0]):
            time.sleep(5)

            # Collect the instances
            instances = []
            for filename in os.listdir(directory): 
                if filename.endswith(".lp"):
                    instances.append(directory + '/' + filename)

            # Sort instance based on approximated hardness
            instances = sorted(instances)
            
            # Result dictionary
            results = {}
            for instance in instances:
                time.sleep(5)
                print(instance.split('/')[-1])
                results[instance.split('/')[-1]] = baseline(main_file, instance, time_limit=60*30)
                
            # Dump a single problem results to a file
            f = open(result_path + main_file.split('/')[0],"wb")
            pickle.dump(results, f)
            f.close()
        

if __name__ == '__main__':
    main()