import os
import sys
import time
import clingo
import pickle
import numpy as np
import matplotlib.pyplot as plt

# Smart enumeration algorithm
from ASPalgorithms.smart_enum import Sapplication

# Does
# 1 - Get all the instances for a problem
# 2 - Run the instances
# 3 - Store results


def collect_problem_intances(directory):
    instances = []
    # Go over all the .lp problem instances in directory
    for filename in os.listdir(directory): 
        if filename.endswith(".lp"):
            instances.append(directory + '/' + filename)
    # Sort instance based on approximated hardness
    instances = sorted(instances)
    return instances


def run_algorithm(main_file, instance, time_limit = 600, model_goal_num = 100):
    
    # Block that runs the clingo enumeration algorithm
    start = time.time()
    smart_arguments = [main_file, instance, "--opt-mode=enum", 
                       model_goal_num, '--time-limit={}'.format(time_limit), "--quiet=1"]
    enum_smart = Sapplication(sys.argv[0])
    clingo.clingo_main(enum_smart, smart_arguments)
    end = time.time()
    
    # Make a result dictionary
    result = {}
    # Weight <-> count pairs
    result["weights"] = enum_smart.result_dict
    # Did we hit the time limit
    result["completed"] = False if time_limit - (end - start) < 0.5 else True
    # Total time used
    result["time"] = end - start
    return result

# Define 
result_path = "Results/"
main_files = ['ConnectedMaximim-densityStillLife/encoding.asp',
              'Hamilton/tsp.asp',
              'FastFood/FastfoodOptimize.asp',
              'BayesianNL/encoding.asp', 
              'MarkovNL/encoding.asp',
              'Supertree/encoding.asp',
              'MaxSat/encoding-show.asp']
directorys = ['ConnectedMaximim-densityStillLife/',
              'Hamilton/instances',
              'FastFood/instances',
              'BayesianNL/instances', 
              'MarkovNL/instances',
              'Supertree/instances',
              'MaxSat/']

def main():
    # 1. Collect
    for main_file, directory in zip(main_files, directorys):
        # Make sure target file exist
        if os.path.isfile(main_file):
            # Sort instance based on approximated hardness
            instances = collect_problem_intances(directory)
            # 2. loop over different num of target models k
            for k in [10, 100, 1000, 10000]:
                # Is there a corresponding result file
                if not os.path.isfile(result_path + main_file.split('/')[0] + "_s_{}".format(k)):
                    results = {}
                    for instance in instances:
                        time.sleep(5)
                        results[instance.split('/')[-1]] = run_algorithm(main_file, instance, time_limit = 60* 30, model_goal_num = k)

                    # 3.Store (k, problem) results pairs to a file
                    f = open(result_path + main_file.split('/')[0] + "_s_{}".format(k), "wb")
                    pickle.dump(results, f)
                    f.close()
    
if __name__ == '__main__':
    main()