import clingo
import copy 
import time
import sys
import re
import itertools

# Observer to track: minimize literals
class Observer:
    def __init__(self):
        self.minimize_literals = {}
        self.priorities = []

    def minimize(self, priority, literals):
        if priority not in self.minimize_literals.keys():
            self.minimize_literals[priority] = []
            self.priorities.append(priority)
        self.minimize_literals[priority] = literals

class Application:
    def __init__(self, name, queue, true_probs, query_true):
        self.program_name = name
        self.queue = queue
        self.true_probs = true_probs
        self.query_true = query_true


    def assignment_probability(self, model):
        node_states = re.findall("record\(([0-9]*),([0-9]*)\)", str(model))
        
        prob = 1
        for node_id, value in node_states:
            prob *= self.true_probs["({},{})".format(node_id, value)]
            
        return prob



    def main(self, ctl, files):
        if len(files) > 0:
            for f in files:
                ctl.load(f)
        else:
            ctl.load("-")

        # ===================================
        # Initialize
        assert(ctl.configuration.solve.opt_mode == 'optN'), 'Set --opt-mode=OptN.'
        start = time.time()
        count = int(ctl.configuration.solve.models)

        # register observer
        obs = Observer()
        ctl.register_observer(obs)

        tmp = []

        # ===================================
        # Ground and Solve
        ctl.ground([("base", [])])
        while count > 0:
            costs = []   
            ctl.configuration.solve.models = count
            with ctl.solve(yield_=True) as handle:
                for model in handle:
                    if model.optimality_proven:
                        costs = model.cost[:]
                        count -= 1

                        probability = self.assignment_probability(model)
                        if count > 0:
                            self.queue.put((probability, model, time.time() - start, costs, self.query_true))
                        
                if handle.get().unsatisfiable:
                    break

            if count > 0:
                with ctl.backend() as backend: 
                    # Rules of
                    # :- aux.
                    # aux :- (cost+1) > sum(literals).
                    for i in range(len(obs.priorities)):
                        aux = backend.add_atom() 
                        backend.add_weight_rule([aux], costs[i] + 1, obs.minimize_literals[obs.priorities[i]])
                        backend.add_rule([], [-aux])

            if (time.time() - start) > (60 * 10): # * 60 * 10):
                break   

        for line in tmp:
            print(line)
    