# ===================================
# A class that holds Bayes networks
# 
#
class node:
    def __init__(self, name, node_id, parents, cpt, states):
        self.id = node_id
        self.name = name
        # List of parents
        self.parents = parents
        # Dictionary of nodes possible assignments
        self.states = states
        # key -> probability
        self.cpt = {}
        prob_sum = 0
        for assignment, prob in cpt:
            key = self.make_assignment_key(assignment)
            self.cpt[key] = prob
            prob_sum += prob

    def make_assignment_key(self, assignment_dictionary):
        key = self.name + ":" + str(assignment_dictionary[self.name])
        for parent in self.parents:
            key += "," + parent + ":" + str(assignment_dictionary[parent])
        return key

    def query(self, assignment):
        # node:A,P1:B,P2:C..
        return self.cpt[self.make_assignment_key(assignment)]


class network:
    def __init__(self):
        self.nodes = {}
        self.net = {}    

    # Appends a node to the network
    def append_node(self, name, parents, cpt, states):
        new_node = node(name, len(self.net), parents, cpt, states)
        self.nodes[name] = len(self.net)
        self.net[name] = new_node

    # Query assignment probability
    def query_assignment_prob(self, assignment): 
        prob = 1
        # Loop over the nodes to find out assignment prob
        for node_name, val in assignment.items():
            prob *= self.net[node_name].query(assignment)
        return prob

    # Return probability for one node
    def query_node_prob(self, node, assignment):
        if all(parent in assignment.keys() for parent in self.net[node].parents):
            return self.net[node].query(assignment)    
        else:
            return 1


    