from pylab import *
import matplotlib.pyplot as plt
import os
import time
import pyAgrum as gum
import pickle
from ASPSampler.solve import clingo_sampler
from utils.network import network
from utils.setup_pyagrum import network_to_pyagrum
import pyAgrum.lib.notebook as gnb


# Number of experiments
loops = 5
result_path = "Results/"

x = []
y = []
run_dictionary = {}
for network, m, K in [("Networks/win95pts.pik", 10, 500)]:
    
    mapper = {}
    with open(network, 'rb') as f:
        network_cpt = pickle.load(f)
        
    # Mapper for real node name -> id
    for i, (key, node) in enumerate(network_cpt.net.items()):
        mapper[key] = i

    # Set up pyAgrum
    bn = network_to_pyagrum(network)
    
    # we want to have evidence size between 1 to 0.5*size(evidence)
    max_ev = int(len(network_cpt.nodes.keys()) * 0.5)
    
    Query_history = []
    
    true_probability = []
    clingo_sampler_probability = []
    Gibbs_sampler_probability = []
    likelihood_weighting_probability = []
    
    time_clingo = []
    time_gibbs = []
    time_exact = []
    
    # Run a desired number of experiments
    for i in range(loops):
        # Info for a single query
        exp_run = {}
        
        # Set up Clingo sampler
        # make a new instantiation to make sure previous threads are done
        clingo_sample = clingo_sampler(network, K=K, m=m)
        
        print("{}/{}".format(i+1, loops), end='\r')
        choices = np.random.choice(list(mapper.keys()), int(np.random.uniform(2, max_ev)), replace=False)

        # Random a query var
        # Random evidence stuff
        query = { choices[0] : 1 }
        evidence =  { }
        for ev in choices[1:]:
            evidence[ev] = np.random.randint(2)
        
        exp_run["query"] = query
        exp_run["evidence"] = evidence
        
        # Approximate Clingo
        # =================================
        start = time.time()
        approx = clingo_sample.query(query, evidence)
        runtime = time.time() - start
        time_clingo.append(runtime)
        # Something went wrong, we did not get any results?
        if len(approx) == 0:
            clingo_prob = -1
        else:
            clingo_prob = approx[-1][1]
        exp_run["clingo"] = {"prob" : clingo_prob, "run" : approx , "time" : runtime}
        # =================================
        
        # Calculate True
        # =================================
        start = time.time()
        ie=gum.LazyPropagation(bn)
        ie.setEvidence(evidence)
        ie.makeInference()
        true_q = ie.posterior(mapper[choices[0]]).tolist()[1]
        runtime = time.time() - start
        time_exact.append(runtime)
        exp_run["true_prob"] = true_q
        exp_run["time_true"] = runtime
        # =================================
        
        
        # Run Pre-implemented Gibbssampler
        # =================================
        start = time.time()
        ie2 = gum.GibbsSampling(bn)
        ie2.setEpsilon(1e-2)
        ie2.setEvidence(evidence)
        ie2.makeInference()
        gibbs_q = ie2.posterior(mapper[choices[0]]).tolist()[1]
        runtime = time.time() - start
        time_gibbs.append(runtime)
        exp_run["Gibbs"] = {"prob" : gibbs_q, "time" : runtime }
        # =================================    
        Query_history.append(exp_run)
    
        # Fast result print
        print("======")
        print(f'Completed run with Clingo apx {exp_run["clingo"]["prob"]:.2f} and true {exp_run["true_prob"]:.2f}')
        print("======\n")
        
    # Finally store all network results to one dictionary
    f = open(result_path + "Run_results_{}.pik".format(network.split('/')[-1]), "wb")
    
    pickle.dump(Query_history, f)
    f.close()
    run_dictionary[network.split('/')[-1]] = Query_history
    
 # Finally store the run dictionary
with open(result_path + "Run_results.pik", "wb") as f:
    pickle.dump(run_dictionary, f)

