import os
import pyAgrum as gum
import pickle

# ====================
# Converts a network from type network.py 
# into pyAgrum instance
#
#
# usage:
#     - Call network_to_pyagrum([network string])
#     - Input network needs to be type of network.py
# ====================

def network_to_pyagrum(network):

    cpt = None
    with open(network, 'rb') as f:
        cpt = pickle.load(f)

    # Set up the network!
    bn=gum.BayesNet('TestNetwork')
    mapper = {}

    # Add all nodes
    for key, node in cpt.net.items():
        tmp = bn.add(gum.LabelizedVariable(key,key,len(node.states)))
        mapper[key] = tmp


    # Add all links!
    for key, node in cpt.net.items():
        for parent in node.parents:
            bn.addArc(*[mapper[parent], mapper[key]])


    # Make the CPT!
    for key, node in cpt.net.items():
        assignments = {}
        others = ""

        # Parse useless lines!
        for key, prob in node.cpt.items():
            tmp = key.split(",")
            self_ass = tmp[0].split(":")[1]
            if int(self_ass) == 1:
                assignments[key] = prob

        # Add CPT lines!
        for key, prob in assignments.items():
            tmp_assign = {}

            tmp = key.split(",")
            self_name = tmp[0].split(":")[0]
            others = tmp[1:]

            for other in others:
                key, val = other.split(":")
                tmp_assign[key] = int(val)

            prob_list = []
            for i in range(len(node.states)):
                allem = [self_name + ":" + str(i)] + others
                prob_list.append(node.cpt[",".join(allem)])

            # Add CPT line
            # prob_list, prob_list[::-1] # [1-prob, prob]
            bn.cpt(mapper[self_name])[tmp_assign] = prob_list  # prob_list[::-1] # [1-prob, prob]
    return bn

        